<?php
namespace Drupal\logsentry\Logger;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Psr\Log\LoggerInterface;

class FileLog implements LoggerInterface {
    use RfcLoggerTrait;

    /**
     * @var \Drupal\Core\File\FileSystemInterface
     */
    protected $fileSystem;

    protected $path = 'temporary://logsentry/';


    public function __construct() {
        $this->fileSystem = \Drupal::service('file_system');
        $this->fileSystem->prepareDirectory($this->path, FileSystemInterface::CREATE_DIRECTORY);
    }

    /**
     * {@inheritdoc}
     */
    public function log($level, \Stringable|string $message, array $context = array()) :void {
        $filename = substr(hash('sha512', microtime()),0, 6);

        $userCurrent = \Drupal::currentUser();
        $user['id'] = $userCurrent->id();
        if($userCurrent->isAuthenticated()) {
            $user['username'] = $userCurrent->getAccountName();
        }

        $data = [
            'level' => $level,
            'message' => $this->getMessage($message, $context),
            'context' => $context,
            'user' => $user,
        ];

        $this->fileSystem->saveData(json_encode($data), $this->path.$filename);
    }

    private function getMessage($message, $context = []) { 
        krsort($context);

        foreach($context as $search => $replace) {
            if(substr($search, 0, 1) == '@') {
                $message = str_replace($search, $replace, $message);
            }         
        }

        return $message;
    }
}

